(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2004-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: layout.ml,v 1.2 2011-02-23 17:43:29 fclement Exp $ *)

type 'a list1 = 'a * 'a list;;

type t =
   | Permutation of int list1
   | C
   | F
;;

let layout_to_ob ob = function
  | C -> Buffer.add_string ob "C"
  | F -> Buffer.add_string ob "F"
  | Permutation (i, is) ->
    let item_to_ob ob i =
      Buffer.add_string ob (string_of_int i);
      Buffer.add_string ob ", " in
    List.iter (item_to_ob ob) (i :: is)
;;

let permutation_of_ib ib =
  let item_of_ib ib = Scanf.bscanf ib "%d" in
  let rec items_of_ib ib accu =
    if Scanf.Scanning.end_of_input ib then List.rev accu else
    Scanf.bscanf ib "%0c" (function
    | '0' .. '9' ->
      item_of_ib ib
        (fun it -> items_of_ib ib (it :: accu))
    | ',' -> Scanf.bscanf ib ",\032" (); items_of_ib ib accu
    | _ -> List.rev accu) in

  Scanf.bscanf ib "%0c" (function
  | '0' .. '9' ->
    item_of_ib ib
     (fun it -> Permutation (it, items_of_ib ib []))
  | c ->
    failwith
      (Printf.sprintf "permutation_of_ib: unexpected char %c" c))
;;

let layout_of_ib ib =
  Scanf.bscanf ib "%0c" (function
  | 'C' -> Scanf.bscanf ib "%_c" C
  | 'F' -> Scanf.bscanf ib "%_c" F
  | '0' .. '9' -> permutation_of_ib ib
  | c ->
    failwith
      (Printf.sprintf "layout_of_ib: unexpected char %c" c))
;;

let check_layout p l =
  if p <= 0 then
    failwith "check_layout: dimension should be strictly positive." else
  match l with
  | C | F -> ()
  | Permutation (i, []) ->
    if i <> 0 then
      failwith
        (Printf.sprintf
           "check_layout: singleton permutation should be [0] not [%d]." i)
  | Permutation (i, l) ->
    let lperm = 1 + List.length l in
    if lperm <> p then
      failwith
        (Printf.sprintf
           "check_layout: permutation size %d \
            and array dimension %d mismatch." lperm p) else
    let v = Array.make p false in
    let check_item i =
      if i < 0 || i >= p then
        failwith
          (Printf.sprintf
             "check_layout: permutation item %d is out of range." i) else
      if v.(i) then
        failwith
          (Printf.sprintf
             "check_layout: permutation item %d found twice." i) in
    List.iter (fun i -> check_item i; v.(i) <- true) (i :: l);;

let check = check_layout;;
let to_ob = layout_to_ob;;
let of_ib = layout_of_ib;;
